REM Description
REM -----------
REM This script creates two HTML files which show Cubase shortcut keys.
REM 1. List of the commands which have keys and their associated key
REM 2. Matrix of all the Keys vs Modifiers, and the commands for each combination
REM The HTML files will be opened in your default browser and are stored in the 
REM   installation folder (see below)


REM Installation
REM ------------
Rem Place this script in the Cubase 'User Settings Data Folder', where the 'Key Commands.xml' file is stored


REM How To Use
REM ----------
Rem Run the script (double click) and it will create two HTML files (in the same folder as the 
REM   script) and display them in your default browser


REM History
REM -------
REM 20150423 - MJV/GargoyleStudio
REM I created this a few years ago from two scripts that I found, so this script 
REM   now creates two HTML pages - sorry, I don't know who originally wrote the scripts!!
REM 20170116 - MJV/GargoyleStudio
REM Updated because the Macro names weren't being shown in the ByKey page, they are now 
REM   shown correctly, with a (m) after to denote that they are a Macro
REM Updated to add a whole new bunch of colours to group each section
REM 20170711 - MJV/GargoyleStudio
REM Corrected the ignoring of the Macro section
REM Added in (ple) for Project Logical Editor commands
REM Added some comments to help understand the code


' TODO: Set the different Const and Arrays to your needs
' ---------------------------------------------------
Option Explicit
Const HTML = True ' True html file, False texte file (Tab delimited)
Const SECTIONNAME = False ' add the section name (True or False)
Const SORTKEY = False ' sort by key name (True or False)
' TODO : to have your keys in a certain order, enter them here:
'-----------------------------------------------
  B = Array( _
    "F1","F2","F3","F4","F5","F6","F7","F8","F9","F10","F11","F12", _
    "`","1","2","3","4","5","6","7","8","9","0", _
    "-","=","_","+","Backspace", _
    "A","B","C","D","E","F","G","H","I","J","K","L", "M", _
    "N","O","P","Q","R","S","T","U","V","W","X","Y","Z", _
    "[", "]","\",";","'",",",".","/","Return","Space", _
    "Insert", "Del", "Home", "End", "PgUp", "PgDown",  _
    "Left Arrow", "Up Arrow", "Right Arrow", "Down Arrow", _
    "Pad0","Pad1","Pad2","Pad3","Pad4","Pad5","Pad6","Pad7", _
    "Pad8","Pad9","Pad .","Pad /","Pad *","Pad -","Pad +","Enter" _
    ) ' end of array
  ' Make sure you have SORTKEY = False ;)
  Const REMOVE_UNUSED = True ' removes unused keys from the list
  ' Find Replace key name , add always by pair: "SX key name", "New Name"
  C = Array("_", "Shift+-", "+", "Shift+=")
  ' C = Array(-1) ' uncomment if you don't want key renaming
  ' Section color (http://msdn.microsoft.com/library/default.asp?url=/workshop/author/dhtml/reference/colors/colors.asp)
  ' Each section name must be followed by a color name
BK = Array( "Mixer",            "linen", _
            "Preferences",      "silver", _
            "Edit",             "hotpink", _
            "Devices",          "coral", _
            "Media",            "darkgray", _
            "MIDI Quantize",    "steelblue", _
            "Set Insert Length","skyblue", _
            "Play Order",       "darkorange", _
            "Transport",        "yellow", _
            "Zoom",             "lightyellow", _
            "Nudge",            "yellow", _
            "Scores Statusbar", "darksalmon", _
            "Score Functions",  "lightsalmon", _
            "Tool",             "aquamarine", _
            "File",             "indianred", _
            "Project",          "darkseagreen", _
            "AddTrack",         "gainsboro", _
            "Editors",          "mistyrose", _
            "Inspector",        "khaki", _
            "Audio",            "thistle", _
            "Automation",       "darkgoldenrod", _
            "Hitpoints",        "lavender", _
            "Audio Realtime Processing",   "lawngreen", _
            "MIDI",             "turquoise", _
            "Navigate",         "burlywood", _
            "Import",           "limegreen", _
            "Export",           "deeppink", _
            "Process",          "lightskyblue", _
            "Process Plugin",   "mediumturquoise", _
            "Analyze",          "gold", _
            "Process Logical Preset",  "mocasin",_
            "Scores",           "orchid", _
            "Score Symbol Palettes", "peru", _
            "Score Align Elements",  "tan", _
            "Score Meter Scale", "yellowgreen", _
            "Workspaces",        "lightyellow", _
            "Macro",             "skyblue", _
            "Windows",           "slateblue", _
            "Window Zones",      "peachpuff" _
            )
  CH = Array("auml" ,228,"ouml" ,246,"uuml" ,252,"szlig" ,223,"Auml" ,196,"Ouml" ,214,"Uuml" ,220,"nbsp" ,160,"Agrave" ,192,"Egrave" ,200,"Eacute" ,201,"Ecirc" ,202,"egrave" ,232,"eacute" ,233,"ecirc" ,234,"agrave" ,224,"iuml" ,239,"ugrave" ,249,"ucirc" ,251,"uuml" ,252,"ccedil" ,231,"AElig" ,198,"aelig" ,330, "OElig" ,338,"oelig" ,339,"euro" ,8364,"laquo" ,171,"raquo" ,187, "sect", 167, "grave", 96)
' / TODO
'---------------------------------------------------
Dim A, B, C, BK, CH
Get_Keys_By_Function
Get_Keys_By_Key

REM --------------------------------
REM --------------------------------
REM --------------------------------
Sub Get_Keys_By_Key
  Dim WshShell, fso, f, f2, f3, dir, dir2, tp, bc, n
  Redim A(UBound(B)+1)
  For f = 0 To UBound(B)
    A(f+1) = Array( B(f), ".", ".", ".", ".", ".", ".", ".", ".")
  Next
  A(0) = Array("key", "-", "Shift", "Ctrl", "Alt", "Ctrl+Shift", "Alt+Shift", "Ctrl+Alt", "Ctrl+Alt+Shift")
  If HTML Then tp = "html" else tp = "txt"
  set WshShell = CreateObject("WScript.Shell")
  Set fso = CreateObject("Scripting.FileSystemObject")
		
  Rem Look for XML file in the current folder
  rem Too specific... dir = WshShell.SpecialFolders("AppData") & "\Steinberg\Cubase SX 3\"
  dim strPath, objFile
  strPath = Wscript.ScriptFullName
  set objFile = fso.GetFile(strPath)
  dir = fso.GetParentFolderName(objFile) & "\"

  Dim outName, inName, tmpName
  outName = "Key Commands By Key." & tp
  inName  = "Key Commands.xml"
  tmpName = "KC_TMP.xml"

  Rem Check if xml file exists first, with tidy abort
  if (NOT fso.FileExists(dir & inName)) Then
	WshShell.Popup("ABORT - Did not find XML file: " & dir & inName)
	WScript.Quit
  end if

  Set f = fso.CreateTextFile(dir & outName, True)
  Set f2 = fso.CreateTextFile(dir & tmpName, True)
  Set f3 = fso.OpenTextFile(dir & inName, 1)

  dim xmlDoc, t, u, v, w, x, y, z, s, d, postfix
  set xmlDoc = CreateObject("Microsoft.XMLDOM")
  dir2 = replace(dir , "\", "/") 
  xmlDoc.validateOnParse = False
  xmlDoc.resolveExternals = False
  xmlDoc.async = False

  ' special foreign chars   
  f2.write "<!DOCTYPE characters [ <!ELEMENT characters (character*) > <!ELEMENT character (#PCDATA ) > "
  For n = 0 To UBound(CH)
   f2.Write "<!ENTITY " & CH(n) & " " & chr(34) & "&#" & CH(n+1) & ";" & chr(34) & " >"
   n = n+1
  Next   
  f2.write "] > "
  f2.write vbCRLF
  f2.write(f3.ReadAll())
  f2.Close
  f3.Close
  xmlDoc.load(dir2 & "KC_TMP.xml")
  If xmlDoc.parseError.errorCode Then
    MsgBox _
    "File: " & dir & "Key Commands.xml" &vbCrLf & _
    "Line : " & xmlDoc.parseError.line & vbCrLf & _
    "Error: " & xmlDoc.parseError.reason, 0, "XML parser error:"
'     Exit Sub
  End If 
  On error resume next

  rem XML Level 1 loop
  rem e.g. <KeyCommands>
  for each t in xmlDoc.childNodes
	 'MsgBox "debug - Level 1 (t) - " & t.attributes.item(0).text
    'If s = "Macros" Then Exit For

    rem Level 2 - Categories or Macros loop
    rem e.g. <list name="Categories" type="list">
    for each u in t.childNodes
      s = u.attributes.item(0).text
	   'MsgBox "debug - u - " & s

      rem Skip the whole Macros section
      If s = "Macros" Then Exit For

      rem Level 3 - always just <item>, not used
      rem e.g. <item>
      for each v in u.childNodes
	     'MsgBox "debug - v - " & v.attributes.item(0).text

        rem Level 4 - Sections level
        rem e.g. <list name="Commands" type="list">
        for each w in v.childNodes
	       'MsgBox "debug - Section (w) - " & w.attributes.item(1).text

          If w.nodename = "string" Then s = w.attributes.item(1).text
          If s = "Process Plugin" Then bc = True Else bc = False
          If s = "Macros" Then Exit For
			 'MsgBox "debug - Section - " & s & " " & d & " " & y.attributes.item(0).text

			 rem Level 5 - Always <item> but within that are 1 or 2 lines
          rem e.g. <item>
          rem        <string name="Name" value="Show/Hide Infoview"/>
          rem        <string name="Key" value="Ctrl+I" wide="true"/>
          rem      <item>
          for each x in w.childNodes
	         'MsgBox "debug - x - " & x.attributes.item(0).text
            If x.childNodes.length = 2 Then

              rem Has 2 lines, so it has a shortcut key defined
              d = x.childNodes.item(0).attributes.item(1).text
              If bc Then
                If len(d) > 34 Then d = Mid(d, 35)
              End If 

              postfix = ""
              if s = "Macro" Then postfix = " (m)"
              if s = "Process Project Logical Editor" Then postfix = " (ple)"
              If x.childNodes.item(1).childNodes.length > 1 Then

                rem May have more than one shortcut key assigned
                for each y in x.childnodes.item(1).childNodes
                  AddKey s, d & postfix, y.attributes.item(0).text
				      'MsgBox "debug - " & s & " " & d & " " & y.attributes.item(0).text
                next
              Else 
                AddKey s, d & postfix, x.childNodes.item(1).attributes.item(1).text
				    'MsgBox "debug - " & s & " " & d & " " & x.childNodes.item(1).attributes.item(1).text
              End If
            End If
          next
        next 
      next
    next 
  next
  If HTML then
    f.write "<TABLE cellSpacing=1 cellPadding=1 width=" & _
     chr(34) & "99%" & chr(34) & "align=center border=1>" & vbCrLf
  End If
  If SORTKEY Then
    For t = 1 To UBound(A)
      For u = t+1 To UBound(A)
       If UCase(A(u)(0)) < UCase(A(t)(0)) Then
         z = A(t) : A(t) = A(u) : A(u) = z
       End If
     Next 
    Next
  End If 
  For t = 0 To UBound(A)
    If HTML Then f.write "<TR>"
    If REMOVE_UNUSED Then
      For B = 1 To UBound(A(t))
        If A(t)(B) <> "." Then Exit For       
      Next
      If B > UBound(A(t)) Then
        Redim b(-1) : A(t) = b
      End If 
    End If
    For u = 0 To UBound(A(t))
      If HTML Then
        If t = 0 Or u = 0 Then
          f.write "<TD align=middle style=" & chr(34) & "FONT-WEIGHT: bold" & Chr(34) & ">" & A(t)(u) & "</TD>"
        Else
          If A(t)(u) = "." Then
            'f.write "<TD align=middle>" & A(t)(u) & "</TD>"
            f.write "<TD align=middle>.</TD>"
          Else 
            f.write "<TD align=middle style=" & chr(34) & "BACKGROUND-COLOR: " & A(t)(u) & "</TD>"
          End If 
        End If 
      Else
        f.write A(t)(u)
        If u <> UBound(A(t)) Then f.write vbTab
      End If
    Next
    If HTML Then f.write "</TR>"
    f.write vbCrLf
  Next
  If HTML then f.write "</TABLE>" & vbCrLf
  f.close
  
  Rem  remoe temp file
  fso.DeleteFile(dir & tmpName)
  
  If HTML Then s = "Explorer.exe " Else s = "notepad.exe "
  WshShell.Run s & dir & outName
End Sub

REM ---------------------
REM ---------------------
REM ---------------------
Sub AddKey(s, d, byval k)
  dim bp, ak, n, kn, bkc, m, ks
  k = Trim(k)
  If IsArray(C) Then
    For n = 0 To UBound(C) Step 2
      If C(n) = k Then
        k = C(n+1)
        Exit for
      End If 
    Next
  End If
  ks = k
  If Right(k, 1) = "+" Then bp = True
  k = Replace(k, "+", vbCr)
  If bp Then k = Left(k, len(k)-1) & "+"
  ak = Split(k, vbCr) : kn = 0
  For n = 0 To UBound(ak)-1
    Select Case UCase(ak(n))
      Case "SHIFT"
        kn = kn Or 1
      Case "CTRL"
        kn = kn Or 2
      Case "ALT"
        kn = kn Or 4
    End Select
  Next
  For n = 1 To UBound(A)
    If UCase(A(n)(0)) = UCase(ak(UBound(ak))) Then Exit For
  Next
  If n > UBOund(A) Then
    Redim Preserve A(n)
    A(n) = Array(ak(UBound(ak)), ".", ".", ".", ".", ".", ".", ".", ".")
  End If
  Select Case kn
    Case 4:
      kn = 3
    Case 3:
      kn = 4
  End Select
  If HTML Then
    bkc = "white"
    If IsArray(BK) Then
      For m = 0 To UBound(BK) Step 2
        If BK(m) = s Then
          bkc = BK(m+1)
          Exit For
        End If
      Next
    End If
  End If 
  If SECTIONNAME Then
    A(n)(kn+1) = bkc &  A(0)(kn) & ">" & s & "-" & d
  Else
    A(n)(kn+1) = bkc & chr(34) & " title=" & chr(34) & "[ " & s & " ]  " & ks & "  : " & d & chr(34) & ">" & d
  End If
End Sub

REM ---------------------------------
REM ---------------------------------
REM ---------------------------------
Sub Get_Keys_By_Function
  Dim WshShell, fso, f, f2, f3, dir, dir2, tp, bc , n
  If HTML Then tp = "html" else tp = "txt"
  set WshShell = CreateObject("WScript.Shell")
  dir = WshShell.SpecialFolders("AppData") & "\Steinberg\Cubase SX 3\"
  Set fso = CreateObject("Scripting.FileSystemObject")
		
		Rem Over-ride dir to be the current folder
		dim strPath, objFile
		strPath = Wscript.ScriptFullName
		set objFile = fso.GetFile(strPath)
		dir = fso.GetParentFolderName(objFile) & "\"

  Dim outName, inName, tmpName
  outName = "Key Commands By Function." & tp
  inName  = "Key Commands.xml"
  tmpName = "KC_TMP.xml"

  Rem Check if xml file exists first
  if (NOT fso.FileExists(dir & inName)) Then
	WshShell.Popup("ABORT - Did not find XML file: " & dir & inName)
	WScript.Quit
  end if
  
  Set f = fso.CreateTextFile(dir & outName, True)
  Set f2 = fso.CreateTextFile(dir & tmpName, True)
  Set f3 = fso.OpenTextFile(dir & inName, 1)

  dim xmlDoc, t, u, v, w, x, y, z, s
  set xmlDoc = CreateObject("Microsoft.XMLDOM")

  dir2 = replace(dir , "\", "/") 
  xmlDoc.validateOnParse = False
  xmlDoc.resolveExternals = False
  xmlDoc.async = False

  ' special foreign chars   
  f2.write "<!DOCTYPE characters [ <!ELEMENT characters (character*) > <!ELEMENT character (#PCDATA ) > "
  For n = 0 To UBound(CH)
   f2.Write "<!ENTITY " & CH(n) & " " & chr(34) & "&#" & CH(n+1) & ";" & chr(34) & " >"
   n = n+1
  Next   
  f2.write "] > "
  f2.write vbCRLF
  f2.write(f3.ReadAll())
  f2.Close
  f3.Close
  xmlDoc.load(dir2 & "KC_TMP.xml")

  If xmlDoc.parseError.errorCode Then
    MsgBox _
    "File: " & dir & "Key Commands.xml" &vbCrLf & _
    "Line : " & xmlDoc.parseError.line & vbCrLf & _
    "Error: " & xmlDoc.parseError.reason, 0, "XML parser error:"
    Exit Sub
  End If 
  On error resume next
  If HTML then f.write "<TABLE cellSpacing=1 cellPadding=1 width=" & _
     chr(34) & "90%" & chr(34) & "align=center border=1>" & vbCrLf
  for each t in xmlDoc.childNodes
    for each u in t.childNodes
      for each v in u.childNodes
        for each w in v.childNodes
          If w.nodename = "string" Then s = w.attributes.item(1).text
          If s = "Process Plugin" Then bc = True Else bc = False
          for each x in w.childNodes
            If x.childNodes.length = 2 Then
              If s <> "" Then
                If HTML Then f.write "<TR><TD align=middle style=" &chr(34) & "FONT-WEIGHT: bold" & chr(34) & ">"
                f.write s : s = ""
                If HTML Then f.write "</TD></TR>"
                f.write vbCrLf
              End If
              If HTML Then f.write "<TR><TD>" Else f.write vbTab
              s = x.childNodes.item(0).attributes.item(1).text
              If bc Then
                If len(s) > 34 Then s = Mid(s, 35)
              End If 
              f.write s : s = ""
              If HTML Then  f.write "</TD>"
              If x.childNodes.item(1).childNodes.length > 1 Then
                If HTML Then f.write "<TD>" else f.write vbTab
                z = False
                for each y in x.childnodes.item(1).childNodes
                  If z Then f.write "  /  " Else z = True
                  f.write y.attributes.item(0).text
                next
                If HTML Then f.write "</TD></TR>"
                f.write vbCrLf
              Else 
                If HTML Then f.write "<TD>" Else f.write vbTab
                f.write x.childNodes.item(1).attributes.item(1).text
                If HTML then f.write "</TD></TR>"
                f.write vbCrLf
              End If
            End If
          next
        next 
      next
    next 
  next
  If HTML then f.write "</TABLE>" & vbCrLf
  f.close
  
  Rem  remoe temp file
  fso.DeleteFile(dir & tmpName)
  
  If HTML Then s = "Explorer.exe " Else s = "notepad.exe "
  WshShell.Run s & dir & outName
End Sub